local i, keystone,_,bag,bagSize

--------------------------------------------------------------------------------------
-- Extracts ilvl and ilvl equiped from the given keystone and puts it in a readable format.
--------------------------------------------------------------------------------------
function KeystoneRollCall:getIlvlLine(keystone)
	if keystone.ilvl and keystone.ilvlEquipped and tonumber(keystone.ilvl) > 0 then
		local dif = keystone.ilvl - keystone.ilvlEquipped
		if dif == 0 then
			return tostring(keystone.ilvlEquipped)
		else
			return keystone.ilvlEquipped .." +"..(keystone.ilvl - keystone.ilvlEquipped)..""
		end
	end
	return ""
end

function KeystoneRollCall:getFullName(unit)
	local name, realm = UnitFullName(unit)
	return name .. "-" .. (realm and realm ~= "" and realm or KCLib:GetPlayerRealm())
end

--------------------------------------------------------------------------------------
-- Get the full name from the given unit address. Assures the realm is always included.
--------------------------------------------------------------------------------------
function KeystoneRollCall:getDungeonName(dungeonId)
	return dungeonId and C_ChallengeMode.GetMapUIInfo(dungeonId) or ""
end

--------------------------------------------------------------------------------------
-- Formats and colors the given dungeon level.
--------------------------------------------------------------------------------------
function KeystoneRollCall:getBestRunLine(bestLevel)
	if bestLevel and bestLevel > 0 then
		return "+" .. self:colorLevelMaxReward(bestLevel,bestLevel)
	end
	return ""
end

--------------------------------------------------------------------------------------
-- Formats and colors the given dungeon level and id.
--------------------------------------------------------------------------------------
function KeystoneRollCall:getBestRunWithIlvlLine(bestLevel, mapId)
	if bestLevel and bestLevel > 0 then
		return "+" .. self:colorLevelMaxReward(bestLevel,bestLevel).. " " .. (mapId and mapId > 0 and ("|cFFFFFFFF "..KeystoneRollCall:getDungeonName(mapId).."|r ") or "") .. self:colorLevelMaxReward("(" .. C_MythicPlus.GetRewardLevelFromKeystoneLevel(bestLevel) .. " ILvl)",bestLevel)
	end
	return ""
end

function KeystoneRollCall:getBestRunLineOffline(bestLevel)
	if bestLevel and bestLevel > 0  then
		return KeystoneRollCall:colorOffline("+" .. bestLevel.. (mapId and mapId > 0 and (" "..KeystoneRollCall:getDungeonName(mapId)) or "") )
	end
	return ""
end

function KeystoneRollCall:getBestRunWithIlvlLineOffline(bestLevel, mapId)
	if bestLevel and bestLevel > 0 then
		return KeystoneRollCall:colorOffline("+" .. bestLevel .. " " .. (mapId and mapId > 0 and (" "..KeystoneRollCall:getDungeonName(mapId).." ") or "") .. "(" .. C_MythicPlus.GetRewardLevelFromKeystoneLevel(bestLevel) .. " ILvl)")
	end
	return ""
end

function KeystoneRollCall:KeystoneToString(keystone)
	if keystone.dummy then
		return KRCLocal:Get("broadcast_no_addon",keystone.name)
	elseif keystone.keystoneLevel == 0 then
		return KRCLocal:Get("broadcast_no_keystone",keystone.name)
	end
	return "+" .. keystone.keystoneLevel .. " "  ..  KeystoneRollCall:getDungeonName(keystone.keystoneDungeonId) .. "  -  " ..  keystone.name
end

function KeystoneRollCall:CompareKeystones(ka,kb)
	local playerName = KCLib:GetFullPlayerName();
	for i = 1, #KeystoneRollCall.sortFields do
		local av
		local bv
		if KeystoneRollCall.sortFields[i][1] == "altOf" then
			if ka.isPlayers then
				av = playerName.. (playerName == ka.name and "A" or "B")
			else
				av = (ka.altOf == ka.name) and (ka.altOf or "") .. "A" or (ka.altOf or ka.name or "") .. "B"	
			end
			if kb.isPlayers then
				bv = playerName.. (playerName == kb.name and "A" or "B")
			else
				bv = (kb.altOf == kb.name) and (kb.altOf or "") .. "A" or (kb.altOf or kb.name or "") .. "B"	
			end
		elseif KeystoneRollCall.sortFields[i][1] == "keystoneDungeonId" then
			av = KeystoneRollCall:getDungeonName(ka.keystoneDungeonId) or "ZZZ"
			bv = KeystoneRollCall:getDungeonName(kb.keystoneDungeonId) or "ZZZ"
		elseif KeystoneRollCall.sortFields[i][1] == "realm" then
			local nameParts =  {strsplit("-",ka.name or"")}
			av = nameParts[2] or""
			nameParts =  {strsplit("-",kb.name or"")}
			bv = nameParts[2] or""
		elseif KeystoneRollCall.sortFields[i][1] == "bestKeystoneLevel" or KeystoneRollCall.sortFields[i][1] == "keystoneLevel" or KeystoneRollCall.sortFields[i][1] == "ilvl"or KeystoneRollCall.sortFields[i][1] == "seasonScore" or KeystoneRollCall.sortFields[i][1] == "ilvlEquipped" then
			av =  tonumber(ka[KeystoneRollCall.sortFields[i][1]]) or 0
			bv =  tonumber(kb[KeystoneRollCall.sortFields[i][1]]) or 0
		else
			av =  ka[KeystoneRollCall.sortFields[i][1]] or ""
			bv =  kb[KeystoneRollCall.sortFields[i][1]] or ""
		end
		
		if av < bv then
			return KeystoneRollCall.sortFields[i][2];
		elseif av > bv then
			return not KeystoneRollCall.sortFields[i][2];
		end
	end
	return 
end