local tab = {}
tab.index = 2000;
tab.isKeyTable = false;
tab.text = KRCLocal:Get("run_history");
tab.value = "run_history";
tab.propertyShowComplete = nil;
tab.propertyShowAlts = nil;
tab.propertyShowKeyless = nil;
tab.propertyShowOffline = nil;
tab.propertyShowLeveling = nil;
tab.propertyShowOpposingFaction = nil;
tab.showFilterResultsEditBox = false;
local testMode = false;

local GREY_BACKGROUND_ICON = "Interface\\AddOns\\KeystoneRollCall\\texture\\GreyBackground";
local BORDER_FILE = [[Interface\PaperDollInfoFrame\UI-GearManager-Title-Background]]
local entryBlocks = {};
local scrollFrame;
local scrollContainer;

local function CreateSpacer(width)
	local label = KeystoneRollCall.AceGUI:Create('Label');
	label:SetText(" ");
	if width <= 0 then
		label:SetFullWidth(true)
	else
		label:SetWidth(width)
	end
	return label
end



local function ShowRunDetails(frame)
	if not testMode then
		return;
	end

	if not GameTooltip.bg then
		GameTooltip.bg = GameTooltip:CreateTexture(nil,"BACKGROUND",nil,1)
		GameTooltip.bg:SetPoint("TOPLEFT",3,-3)
		GameTooltip.bg:SetPoint("BOTTOMRIGHT",-3,3)
		GameTooltip.bg:SetColorTexture(0,0,0,1)
	end
	
	GameTooltip_SetDefaultAnchor( GameTooltip, UIParent )
	GameTooltip:SetText("Not Yet Implemented.")
	GameTooltip:AddLine(" ")
	
	local runInfo = frame.runInfo;
	if runInfo then
		for i, partyMember in ipairs(runInfo.partyMembers) do
			GameTooltip:AddLine("	" .. partyMember.name .. ": " .. partyMember.guid);
		end
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine("Found Run Info!");
		if runInfo.combatLog then
			GameTooltip:AddLine("Found Combat Log!!!");
			for guid, playerLog in pairs(runInfo.combatLog) do
				GameTooltip:AddLine("	" .. guid .. ": " .. playerLog.totalDamage);
			end
		end
	end
	-- local damageMeter = {}
	-- for i, logEntry in ipairs(runInfo.damageLog) do
		-- print(logEntry.sourceGUID,logEntry.damage)
		-- damageMeter[logEntry.sourceGUID] = (damageMeter[logEntry.sourceGUID] or 0) + (tonumber(logEntry.damage) or 0);
	-- end
	
	-- for guid, damage in pairs(damageMeter) do
		-- GameTooltip:AddDoubleLine(guid, damage);
	-- end
	
	GameTooltip.bg:Show()
	GameTooltip:Show();
	GameTooltip:ClearAllPoints();
	GameTooltip:SetPoint("TOPLEFT",KeystoneRollCall.frame.frame,"TOPRIGHT",-5,-2);
	GameTooltip:SetPoint("TOPRIGHT",KeystoneRollCall.frame.frame,"TOPRIGHT",297,0);
	GameTooltip:SetPoint("BOTTOMLEFT",KeystoneRollCall.frame.frame,"BOTTOMRIGHT",0,5);
end

local function HideRunDetails(frame)
	if GameTooltip.bg then
		GameTooltip.bg:Hide()
	end
	GameTooltip:Hide()
end


local function GetEntryBlock(index)
	if not entryBlocks[index] then
		local entryBlock = KeystoneRollCall.AceGUI:Create("SimpleGroup");
		entryBlocks[index] = entryBlock;
		entryBlock.index = index;
		entryBlock.frame:SetMovable(false);
		entryBlock:SetFullWidth(true);
		entryBlock:SetLayout(nil);
		entryBlock:SetHeight(80);
		entryBlock.frame:SetScript("OnEnter", ShowRunDetails);
		entryBlock.frame:SetScript("OnLeave", HideRunDetails);
	
		local backgroundIcon = KeystoneRollCall.AceGUI:Create("Icon");
		backgroundIcon:SetPoint("RIGHT",entryBlock.frame,"RIGHT",0,0);
		backgroundIcon:SetImage(BORDER_FILE);
		backgroundIcon:SetImageSize(1100,82);
		backgroundIcon.frame:SetFrameStrata("BACKGROUND");
		backgroundIcon.frame:SetScript("OnEnter", ShowRunDetails);
		backgroundIcon.frame:SetScript("OnLeave", HideRunDetails);
		entryBlock:AddChild(backgroundIcon);
		entryBlock.backgroundIcon = backgroundIcon;
		
		local dungeonIcon = KeystoneRollCall.AceGUI:Create("Icon");
		dungeonIcon:SetPoint("LEFT",entryBlock.frame,"LEFT",-5,0);
		dungeonIcon:SetImageSize(80,80);
		dungeonIcon.frame:SetScript("OnEnter", ShowRunDetails);
		dungeonIcon.frame:SetScript("OnLeave", HideRunDetails);
		entryBlock:AddChild(dungeonIcon);
		entryBlock.dungeonIcon = dungeonIcon;
		
		local resultsLabel = KeystoneRollCall.AceGUI:Create("Label");
		resultsLabel:SetPoint("RIGHT",entryBlock.frame,"RIGHT",5,5);
		resultsLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 60, "OUTLINE");
		resultsLabel:SetJustifyH("right");
		resultsLabel:SetJustifyV("CENTER");
		resultsLabel:SetHeight(80);
		resultsLabel:SetWidth(90);
		entryBlock:AddChild(resultsLabel);
		entryBlock.resultsLabel = resultsLabel;
		
		local levelLabel = KeystoneRollCall.AceGUI:Create("Label");
		levelLabel:SetPoint("LEFT",entryBlock.frame,"LEFT",-5,0);
		levelLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 30, "OUTLINE");
		levelLabel:SetColor(1,0.8196079,0,1);
		levelLabel:SetJustifyH("CENTER");
		levelLabel:SetJustifyV("CENTER");
		levelLabel:SetHeight(80);
		levelLabel:SetWidth(110);
		entryBlock:AddChild(levelLabel);
		entryBlock.levelLabel = levelLabel;
	
		local modifierLabel = KeystoneRollCall.AceGUI:Create("Label");
		modifierLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 16, "OUTLINE");
		modifierLabel:SetColor(1,0.8196079,0,1);
		modifierLabel:SetPoint("BOTTOMLEFT",entryBlock.frame,"BOTTOMLEFT",0,0);
		modifierLabel:SetJustifyH("CENTER");
		modifierLabel:SetJustifyV("CENTER");
		modifierLabel:SetHeight(80);
		modifierLabel:SetWidth(100);
		entryBlock:AddChild(modifierLabel);
		entryBlock.modifierLabel = modifierLabel;
	
		local summaryLabel = KeystoneRollCall.AceGUI:Create("Label");
		summaryLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 11.5, "OUTLINE");
		summaryLabel:SetColor(1,0.8196079,0,1);
		summaryLabel:SetPoint("TOP",entryBlock.frame,"TOP",0,-5);
		summaryLabel:SetJustifyH("CENTER");
		summaryLabel:SetJustifyV("CENTER");
		summaryLabel:SetWidth(500);
		entryBlock:AddChild(summaryLabel);
		entryBlock.summaryLabel = summaryLabel;
		
		local deathsLabel = KeystoneRollCall.AceGUI:Create("Label")
		deathsLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "OUTLINE");
		deathsLabel:SetColor(1,0.8196079,0,1);
		deathsLabel:SetPoint("BOTTOMLEFT",entryBlock.frame,"BOTTOMLEFT",100,5);
		deathsLabel:SetJustifyH("LEFT");
		deathsLabel:SetJustifyV("CENTER");
		entryBlock:AddChild(deathsLabel);
		entryBlock.deathsLabel = deathsLabel;
		
		local dateLabel = KeystoneRollCall.AceGUI:Create("Label");
		dateLabel:SetFont(KeystoneRollCall.db.global.primaryFontPath, 12, "OUTLINE");
		dateLabel:SetColor(1,0.8196079,0,1);
		dateLabel:SetPoint("BOTTOMRIGHT",entryBlock.frame,"BOTTOMRIGHT",0,5);
		dateLabel:SetJustifyH("RIGHT");
		dateLabel:SetJustifyV("CENTER");
		entryBlock:AddChild(dateLabel);
		entryBlock.dateLabel = dateLabel;		
	
		for z = 1,5 do
			local classIcon = KeystoneRollCall.AceGUI:Create("Icon");
			classIcon:SetPoint("LEFT",entryBlock.frame,"LEFT",-49 + (100*z),0);
			classIcon:SetImageSize(20,20);
			classIcon.frame:SetScript("OnEnter", ShowRunDetails);
			classIcon.frame:SetScript("OnLeave", HideRunDetails);
			entryBlock:AddChild(classIcon);
			entryBlock["classIcon"..z] = classIcon;	
			
			local memberName = KeystoneRollCall.AceGUI:Create("Label");
			memberName:SetFont(KeystoneRollCall.db.global.primaryFontPath, 11, "OUTLINE");
			memberName:SetPoint("LEFT",entryBlock.frame,"LEFT",18 + (100*z),0);
			memberName.frame:SetScript("OnEnter", ShowRunDetails);
			memberName.frame:SetScript("OnLeave", HideRunDetails);
			memberName:SetJustifyH("LEFT");
			memberName:SetJustifyV("CENTER");
			entryBlock:AddChild(memberName);
			entryBlock["memberName"..z] = memberName;
		end
	
		scrollFrame:AddChild(entryBlock,entryBlocks[index-1]);		
	end
	entryBlocks[index].frame:Show()
	return entryBlocks[index];
end


tab.setupContainer = function(container)
	scrollContainer = KeystoneRollCall.AceGUI:Create("SimpleGroup") -- "InlineGroup" is also good
	scrollContainer:SetFullWidth(true)
	scrollContainer:SetHeight(398)
	scrollContainer:SetLayout("Fill") -- important!
	container:AddChild(scrollContainer)

	scrollFrame = KeystoneRollCall.AceGUI:Create("ScrollFrame")
	scrollFrame:SetLayout("Flow") -- probably?
	scrollContainer:AddChild(scrollFrame)

	entryBlocks = {};
	
	for i, runInfo in ipairs(KCLib:GetPlayerKeystone().runHistory) do
		if runInfo.dungeonId > 0 then 
			local entryBlock = GetEntryBlock(i);
			local score = KCLib:GetRunScore(runInfo.dungeonId, runInfo.level, runInfo.completionTime);
			local name, _, timeLimit = C_ChallengeMode.GetMapUIInfo(runInfo.dungeonId);
			
			entryBlock.frame.runInfo = runInfo
			entryBlock.backgroundIcon.frame.runInfo = runInfo
			entryBlock.dungeonIcon.frame.runInfo = runInfo
			
			entryBlock.dungeonIcon:SetImage(select(4,C_ChallengeMode.GetMapUIInfo(runInfo.dungeonId)));
			entryBlock.levelLabel:SetText(runInfo.level or 0);
			entryBlock.modifierLabel:SetText("+"..runInfo.enemyDamagerModifier .. "%");
			entryBlock.deathsLabel:SetText(KRCLocal:Get("run_death_count",runInfo.deathCount, runInfo.deathTimePenalty));
			entryBlock.dateLabel:SetText(date("%m/%d/%y",runInfo.timestamp));
			
			for z = 1,5 do
				local partyMember = runInfo.partyMembers[z];
				if partyMember then
					local classInfo = C_CreatureInfo.GetClassInfo(partyMember.classID);
					entryBlock["classIcon"..z]:SetImage("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES",unpack(CLASS_ICON_TCOORDS[classInfo.classFile])); 	
					entryBlock["memberName"..z]:SetColor(GetClassColor(classInfo.classFile));
					entryBlock["memberName"..z]:SetText(partyMember.name);
					entryBlock["classIcon"..z].frame.runInfo = runInfo
					entryBlock["memberName"..z].frame.runInfo = runInfo
				else
					entryBlock["classIcon"..z]:SetImage(nil);
					entryBlock["memberName"..z]:SetText("");
				end
			end
			
			if runInfo.keystoneUpgradeLevels >= 3 then
				entryBlock.resultsLabel:SetColor(1.0,0.5,0.0,1);
				entryBlock.resultsLabel:SetText("+3");
			elseif runInfo.keystoneUpgradeLevels >= 2 then
				entryBlock.resultsLabel:SetColor(0.64,0.21,0.93);
				entryBlock.resultsLabel:SetText("+2");
			elseif runInfo.keystoneUpgradeLevels >= 1 then
				entryBlock.resultsLabel:SetColor(0.12,1.0,0.0);
				entryBlock.resultsLabel:SetText("+1");
			else
				entryBlock.resultsLabel:SetColor(0.62,0.62,0.62,1);
				entryBlock.resultsLabel:SetText("-1");
			end
			
			entryBlock.summaryLabel:SetText(KRCLocal:Get("run_summary",name,runInfo.level,KeystoneRollCall:FormatTimeString(runInfo.completionTime),KeystoneRollCall:FormatTimeString(timeLimit),score))
			entryBlock.frame:Show()
		end
	end
	KeystoneRollCall.frame.statusBox:SetText(KRCLocal:Get("run_found_count",#KCLib:GetPlayerKeystone().runHistory))
	scrollFrame:DoLayout();
	KeystoneRollCall.frame:DoLayout();
	KeystoneRollCall:RefreshTab();
end


--KeystoneRollCall.tabs[tab.value] = tab;