KeystoneRollCall = LibStub('AceAddon-3.0'):NewAddon('KeystoneRollCall', 'AceEvent-3.0', 'AceConsole-3.0','AceTimer-3.0');
_G.KeystoneRollCall = KeystoneRollCall;
KeystoneRollCall.tabs = {}
KeystoneRollCall.AceGUI = LibStub('AceGUI-3.0');	
KeystoneRollCall.Registry = LibStub('AceConfigRegistry-3.0')
KeystoneRollCall.ScrollingTable = LibStub('ScrollingTable');
KeystoneRollCall.MAX_REWARD_LEVEL = 10;
KeystoneRollCall.sortFields = {{"name",true}}
KeystoneRollCall.version = GetAddOnMetadata("KeystoneRollCall", "Version");
KeystoneRollCall.newestVersionFound = KeystoneRollCall.version;
KeystoneRollCall.runInProgress = false;

local _,i
local MSG_PREFIX = "KRC-MSG"
local pendingKeystoneEntries = {}

KeystoneRollCall.columnSort = function (table, rowa, rowb, sortbycol)
	local field = table.cols[sortbycol].field or table.cols[sortbycol].name:lower();
	local asc = table.cols[sortbycol].sort == "asc"
	local ka = KeystoneRollCall.tableData[rowa][1]
	local kb = KeystoneRollCall.tableData[rowb][1]
	
	if KeystoneRollCall.sortFields[1][1] == field then
		KeystoneRollCall.sortFields[1][2] = asc
	else
		for i = 2, #KeystoneRollCall.sortFields do
			if KeystoneRollCall.sortFields[i] and KeystoneRollCall.sortFields[i][1] == field then
				tremove(KeystoneRollCall.sortFields,i)
			end
		end
		tinsert(KeystoneRollCall.sortFields,1,{field,true})
	end
	
	return KeystoneRollCall:CompareKeystones(ka, kb);
end

function dump(o)
   if type(o) == 'table' then
      local s = '{ '
      for k,v in pairs(o) do
         if type(k) ~= 'number' then k = '"'..k..'"' end
         s = s .. '['..k..'] = ' .. dump(v) .. ','
      end
      return s .. '} '
   else
      return tostring(o)
   end
end

function KeystoneRollCall:SuggestAddon(channel,target)
	ChatThrottlePlusLib:SendChatMessage("NORMAL", MSG_PREFIX, KRCLocal:GetRandomSuggestion(), channel, "Common", target)
end

function KeystoneRollCall:DoSort()
	if not KeystoneRollCall.tableData then
		return
	end
	table.sort(KeystoneRollCall.tableData, function(ka,kb) 
		return self:CompareKeystones(ka[1],kb[1])
	end);
end

function KeystoneRollCall:BroadcastKey(keystone,channel,target)
	ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX,self:KeystoneToString(keystone), channel, "Common", target)
end

function KeystoneRollCall:BroadcastListedKeys(channel,target)
	self:DoSort()
	if not KeystoneRollCall.tableData or #KeystoneRollCall.tableData == 0 then
		print(KRCLocal:Get("no_keystone_broadcast"));
	else
		ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX,KRCLocal:Get("title").." (" .. #KeystoneRollCall.tableData .. "):",channel, "Common",target)
		for i,entry in ipairs(KeystoneRollCall.tableData) do
			if i > 25 then
				break;
			end
			ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX,"     " .. self:KeystoneToString(entry[1]), channel, "Common", target)
		end
		if #KeystoneRollCall.tableData > 25 then
			ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX,KRCLocal:Get("title",#KeystoneRollCall.tableData-25), channel, "Common", target)
		end
	end	
end

function KeystoneRollCall:OnInitialize() 
	self:RegisterChatCommand("krc", "SlashCommand")
	self:RegisterSettings()
	if KCLib:GetSeasonID() == 4 then
		self.MAX_REWARD_LEVEL = 15;
	else
		self.MAX_REWARD_LEVEL = 10;
	end
end

function KeystoneRollCall:SlashCommand(args)
	
	local argArray = {strsplit(" ",args)}
	if argArray[1] == "version" then
		print(KeystoneRollCall.version);
	elseif argArray[1] == "dump" then
		local runType = argArray[2]
		local dungeonId = tonumber(argArray[3]) or 0
		if not C_ChallengeMode.GetMapUIInfo(dungeonId) then
			print("No dungeon found for ID: " .. dungeonId )
			return
		end
		
		local keystone = KCLib:GetPlayerKeystone()
		if runType == "season" then
			print(dump(keystone.seasonRunInfos[dungeonId]));
		elseif runType == "week" then
			print(dump(keystone.weekRunInfos[dungeonId]));
		else
			print("Run type should be 'season' or 'week', not: " .. runType)
		end
	elseif argArray[1] == "ClearAll" then
		print("Removing all keystone entries, keystones will resume being collected momentarily.")
		KeystoneRollCall:ClearKeystoneEntries()
		KCLib:SetPlayerKeystone(KeystoneRollCall:GetPlayerKeystoneEntry())
		KCLib:RequestFullUpdate()
	else
		if not KeystoneRollCall.frame or not KeystoneRollCall.frame:IsVisible() then
			KeystoneRollCall:Show()
		end
	end
end

KCLib:AddKeystoneListener(function(keystoneEntry,channel,sender)
	KRCLogger:LogKeystoneReceived(keystoneEntry,channel,sender)
	keystoneEntry.isPlayer = channel == "PLAYER";
	tinsert(pendingKeystoneEntries,keystoneEntry);
end)

function KeystoneRollCall:DoAnnouncements(keystoneEntry, oldKeystoneEntry)
	if KeystoneRollCall.runInProgress then
		return;
	end
	if keystoneEntry.keystoneLevel and keystoneEntry.keystoneLevel > 0 and (not oldKeystoneEntry or oldKeystoneEntry.keystoneLevel ~= keystoneEntry.keystoneLevel or oldKeystoneEntry.keystoneDungeonId ~= keystoneEntry.keystoneDungeonId) then
		if keystoneEntry.isPlayer then
			if KeystoneRollCall.db.global.printPlayerNewKey then
				print(KRCLocal:Get("announce_new_keystone_player").." +" ..  KeystoneRollCall:colorLevelDifficulty(keystoneEntry.keystoneLevel,keystoneEntry.keystoneLevel) .. " "..KeystoneRollCall:getDungeonName(keystoneEntry.keystoneDungeonId) )
			end
		
			if KeystoneRollCall.db.global.partyPlayerNewKey and IsInGroup() then
				ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX, KRCLocal:Get("announce_new_keystone",UnitName("Player")).." +" .. keystoneEntry.keystoneLevel .. " " .. KeystoneRollCall:getDungeonName(keystoneEntry.keystoneDungeonId), "PARTY", "Common")
			end
			
			if KeystoneRollCall.db.global.guildPlayerNewKey and GetGuildInfo("player") then
				ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX, KRCLocal:Get("announce_new_keystone",UnitName("Player")).." +" .. keystoneEntry.keystoneLevel .. " " .. KeystoneRollCall:getDungeonName(keystoneEntry.keystoneDungeonId), "GUILD", "Common")
			end
		elseif KeystoneRollCall.db.global.printOthersNewKeys then
			print(KRCLocal:Get("announce_new_keystone",KeystoneRollCall:colorClass(keystoneEntry.name,keystoneEntry.class)).." +" ..  KeystoneRollCall:colorLevelDifficulty(keystoneEntry.keystoneLevel,keystoneEntry.keystoneLevel) .. " "..KeystoneRollCall:getDungeonName(keystoneEntry.keystoneDungeonId));
		end
	end
	
	if keystoneEntry.bestKeystoneLevel > 0 and (not oldKeystoneEntry or oldKeystoneEntry.bestKeystoneLevel ~= keystoneEntry.bestKeystoneLevel) then
		if keystoneEntry.isPlayer then
			if KeystoneRollCall.db.global.printPlayerBestRun then
				print(KRCLocal:Get("announce_new_record_player").." +" .. keystoneEntry.bestKeystoneLevel .. " "..KeystoneRollCall:getDungeonName(keystoneEntry.bestKeystoneDungeonId).."" )
			end
			
			if KeystoneRollCall.db.global.partyPlayerBestRun and IsInGroup() then
				ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX,KRCLocal:Get("announce_new_record",UnitName("Player")).." +" .. keystoneEntry.bestKeystoneLevel .. " " .. KeystoneRollCall:getDungeonName(keystoneEntry.bestKeystoneDungeonId), "PARTY", "Common")
			end
			
			if KeystoneRollCall.db.global.guildPlayerBestRun and GetGuildInfo("player") then
				ChatThrottlePlusLib:SendChatMessage("NORMAL",MSG_PREFIX,KRCLocal:Get("announce_new_record",UnitName("Player")).." +" .. keystoneEntry.bestKeystoneLevel .. " " .. KeystoneRollCall:getDungeonName(keystoneEntry.bestKeystoneDungeonId), "GUILD", "Common")
			end
		elseif KeystoneRollCall.db.global.printOthersBestRuns then
			print(KRCLocal:Get("announce_new_record",KeystoneRollCall:colorClass(keystoneEntry.name,keystoneEntry.class)).." +" .. KeystoneRollCall:colorLevelDifficulty(keystoneEntry.bestKeystoneLevel,keystoneEntry.bestKeystoneLevel) .. " "..KeystoneRollCall:getDungeonName(keystoneEntry.bestKeystoneDungeonId).."" );
		end
	end
end


function KeystoneRollCall:FormatTimeString(timeAmount)
	local seconds, ms;

	local hours = floor(timeAmount / 3600);
	local minutes = floor((timeAmount / 60) - (hours * 60));
	seconds = timeAmount - hours * 3600 - minutes * 60;
	if hours > 0 then
		return format("%02d:%02d:%02d", hours, minutes, seconds);
	else
		return format("%02d:%02d", minutes, seconds);
	end
end

local function EventHandlerFunction(self, event, ...)
	if event == "CHALLENGE_MODE_RESET" or event == "CHALLENGE_MODE_COMPLETED" then
		KeystoneRollCall.runInProgress = false;
	elseif event == "CHALLENGE_MODE_START" then
		KeystoneRollCall.runInProgress = true;
	end
end

local eventHandler = CreateFrame("Frame", nil, WorldFrame)
eventHandler:SetScript("OnEvent", EventHandlerFunction)
eventHandler:RegisterEvent("PLAYER_LOGIN")
eventHandler:RegisterEvent("CHALLENGE_MODE_START")
eventHandler:RegisterEvent("CHALLENGE_MODE_RESET")
eventHandler:RegisterEvent("CHALLENGE_MODE_COMPLETED")

-- Process keystone entries that have been received in a controlled queue, no more then 1 per frame.
eventHandler:SetScript("OnUpdate", function()  
	if #pendingKeystoneEntries > 0 then
		local keystoneEntry = tremove(pendingKeystoneEntries,1);
		if keystoneEntry then
			local oldKeystoneEntry = KeystoneRollCall:SaveKeystoneEntry(keystoneEntry);
			if not keystoneEntry.isPlayer or keystoneEntry.name == keystoneEntry.altOf then
				KeystoneRollCall:DoAnnouncements(keystoneEntry, oldKeystoneEntry);
			end
		end
	end
end)